/**-----------------------------------------------------------------------------------------
* Copyright © 2023 Progress Software Corporation. All rights reserved.
* Licensed under commercial license. See LICENSE.md in the project root for more information
*-------------------------------------------------------------------------------------------*/
import { OnDestroy, Renderer2, ElementRef, EventEmitter, NgZone, OnInit, ChangeDetectorRef } from "@angular/core";
import { FileRestrictions, FileInfo } from './types';
import { LocalizationService } from '@progress/kendo-angular-l10n';
import { UploadService } from './upload.service';
import { FileMap } from './types/file-map';
import { Direction } from './types/direction';
import { RemoveEvent, SelectEvent } from './events';
import { NavigationService } from './navigation.service';
import { FileTemplateDirective } from './templates/file-template.directive';
import { DropZoneService } from './dropzone.service';
import { FileInfoTemplateDirective } from './templates/file-info-template.directive';
import * as i0 from "@angular/core";
/**
 * @hidden
 */
export declare const FILESELECT_VALUE_ACCESSOR: any;
export declare class FileSelectComponent implements OnInit, OnDestroy {
    private uploadService;
    private localization;
    private navigation;
    private dropZoneService;
    private ngZone;
    private renderer;
    private cdr;
    /**
     * Sets the `accept` attribute of the `input` element of the FileSelect.
     */
    accept: string;
    /**
     * Disables the FileSelect.
     * The default value is `false`.
     */
    disabled: boolean;
    /**
     * Enables the selection of multiple files
     * ([see example]({% slug fileprocessing_upload %}#toc-upload-of-sinlge-or-multiple-files)).
     * If set to `false`, only one file can be selected at a time.
     */
    multiple: boolean;
    /**
     * Sets the `name` attribute of the `input` element of the FileSelect.
     */
    set name(name: string);
    get name(): string;
    /**
     * Toggles the visibility of the file list.
     */
    showFileList: boolean;
    /**
     * Specifies the [`tabindex`](https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/tabindex) of the FileSelect.
     */
    tabindex: number;
    /**
     * Sets the restrictions for selected files.
     */
    set restrictions(restrictions: FileRestrictions);
    get restrictions(): FileRestrictions;
    /**
     * Specifies the id of the external drop zone to associate with the FileSelect.
     */
    zoneId: string;
    /**
     * @hidden
     */
    focusableId: string;
    fileTemplate: FileTemplateDirective;
    fileInfoTemplate: FileInfoTemplateDirective;
    fileSelectInput: ElementRef;
    fileSelectButton: ElementRef;
    /**
     * Fires when the user navigates outside the component.
     */
    onBlur: EventEmitter<any>;
    /**
     * Fires when the component is focused.
     */
    onFocus: EventEmitter<any>;
    /**
     * Fires when files are selected. If prevented, the selected files will not be added to the list.
     */
    select: EventEmitter<SelectEvent>;
    /**
     * Fires when a file is about to be removed. If prevented, the file will remain in the list.
     */
    remove: EventEmitter<RemoveEvent>;
    /**
     * Fires when the value of the component has changed as a result of a successful `select` or `remove` operation.
     */
    valueChange: EventEmitter<Array<File>>;
    hostDefaultClasses: boolean;
    get hostDisabledClass(): boolean;
    get dir(): string;
    get hostRole(): string;
    /**
     * @hidden
     */
    _restrictions: FileRestrictions;
    fileList: FileMap;
    direction: Direction;
    wrapper: HTMLElement;
    inputElementId: string;
    private documentClick;
    private blurSubscription;
    private wrapperFocusSubscription;
    private selectButtonFocusSubscription;
    private localizationChangeSubscription;
    private subs;
    constructor(uploadService: UploadService, localization: LocalizationService, navigation: NavigationService, dropZoneService: DropZoneService, ngZone: NgZone, renderer: Renderer2, cdr: ChangeDetectorRef, wrapper: ElementRef);
    ngOnInit(): void;
    /**
     * @hidden
     */
    textFor(key: string): string;
    /**
     * Focuses the underlying input element.
     */
    focus(): void;
    ngOnDestroy(): void;
    /**
     * @hidden
     */
    writeValue(newValue: any): void;
    protected onTouchedCallback: Function;
    protected onChangeCallback: Function;
    /**
     * @hidden
     */
    registerOnChange(fn: any): void;
    /**
     * @hidden
     */
    registerOnTouched(fn: any): void;
    /**
     * @hidden
     */
    setDisabledState(isDisabled: boolean): void;
    /**
     * Removes specific file from the file list.
     */
    removeFileByUid(uid: string): void;
    /**
     * Visually clears all files from the UI.
     */
    clearFiles(): void;
    /**
     * @hidden
     * Used to determine if the component is empty.
     */
    isEmpty(): boolean;
    /**
     * @hidden
     * Used by the external dropzone to add files to the FileSelect
     */
    addFiles(files: FileInfo[]): void;
    /**
     * @hidden
     */
    get selectButtonTabIndex(): number;
    /**
     * @hidden
     */
    getIds(): {
        buttonId: string;
        inputId: string;
    };
    /**
     * @hidden
     */
    onFileSelectButtonFocus(_event?: any): void;
    /**
     * @hidden
     */
    onFileSelectButtonBlur(_event?: any): void;
    private subscribeBlur;
    private subscribeFocus;
    private handleKeydown;
    private attachEventHandlers;
    private setDefaultSettings;
    static ɵfac: i0.ɵɵFactoryDeclaration<FileSelectComponent, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<FileSelectComponent, "kendo-fileselect", ["kendoFileSelect"], { "accept": "accept"; "disabled": "disabled"; "multiple": "multiple"; "name": "name"; "showFileList": "showFileList"; "tabindex": "tabindex"; "restrictions": "restrictions"; "zoneId": "zoneId"; "focusableId": "focusableId"; }, { "onBlur": "blur"; "onFocus": "focus"; "select": "select"; "remove": "remove"; "valueChange": "valueChange"; }, ["fileTemplate", "fileInfoTemplate"], never>;
}
